/*
    AttendanceManager
    Copyright (C) 2010  Victor Carceler

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
 */

package org.iespuigcastellar.attendancemanager.core;

import java.io.ByteArrayInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import com.vaadin.Application;
import com.vaadin.terminal.DownloadStream;
import com.vaadin.terminal.StreamResource;

/**
 * Cuando se descarga un DownloadResource es importante que tenga un nombre único,
 * si no es así el navegador puede ignorar las cabeceras http de nocache y
 * mostrar una versión antigua.
 */
@SuppressWarnings("serial")
public class DownloadResource extends StreamResource {
	

	    private final String filename;
	    private final String mimeType;

	    public DownloadResource(ByteArrayInputStream source, Application application, String filename, String mimeType) throws FileNotFoundException {
	        super(new FileStreamResource(source), filename, application);
	        this.filename = filename;
	        this.mimeType = mimeType;
	        //setCacheTime(1);
	        //setBufferSize(-1);
	    }

	    public DownloadStream getStream() {
	        DownloadStream stream = new DownloadStream(getStreamSource().getStream(), mimeType, filename);
	        stream.setParameter("Content-Disposition", "attachment;filename=" + filename);
	        //stream.setCacheTime(-1);
	        return stream;
	    }

	    private static class FileStreamResource implements StreamResource.StreamSource {

	        private final InputStream inputStream;

	        public FileStreamResource(ByteArrayInputStream source) throws FileNotFoundException {
	            //inputStream = new FileInputStream(fileToDownload);
	        	inputStream = source;
	        }

	        public InputStream getStream() {
	            return inputStream;
	        }
	    }
	


    
    /*
     * public class DownloadResource extends StreamResource {

    private final String filename;

    public DownloadResource(File fileToDownload, Application application)
            throws FileNotFoundException {
        super(new FileStreamResource(fileToDownload), fileToDownload.getName(),
                application);

        this.filename = fileToDownload.getName();
    }

    public DownloadStream getStream() {
        DownloadStream stream = new DownloadStream(getStreamSource()
                .getStream(), "application/pdf", filename);
        stream.setParameter("Content-Disposition", "attachment;filename="
                + filename);
        return stream;
    }

    private static class FileStreamResource implements
            StreamResource.StreamSource {

        private final InputStream inputStream;

        public FileStreamResource(File fileToDownload)
                throws FileNotFoundException {
            inputStream = new FileInputStream(fileToDownload);
        }

        public InputStream getStream() {
            return inputStream;
        }
    }
     */
}