/*
    AttendanceManager
    Copyright (C) 2010  Victor Carceler

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
 */

package org.iespuigcastellar.attendancemanager.screenlayouts;

import java.text.DateFormat;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.ResourceBundle;
import java.util.Set;

import org.iespuigcastellar.attendancemanager.AttendancemanagerApplication;
import org.iespuigcastellar.attendancemanager.core.Constants;
import org.iespuigcastellar.attendancemanager.core.DatabaseStorage;
import org.iespuigcastellar.attendancemanager.core.Group;
import org.iespuigcastellar.attendancemanager.core.Logger;
import org.iespuigcastellar.attendancemanager.core.Miss;
import org.iespuigcastellar.attendancemanager.core.Student;
import org.iespuigcastellar.attendancemanager.core.Term;

import com.vaadin.data.Item;
import com.vaadin.data.Property;
import com.vaadin.data.Property.ValueChangeEvent;
import com.vaadin.data.util.IndexedContainer;
import com.vaadin.data.util.QueryContainer;
import com.vaadin.terminal.ThemeResource;
import com.vaadin.ui.Alignment;
import com.vaadin.ui.Button;
import com.vaadin.ui.ComboBox;
import com.vaadin.ui.GridLayout;
import com.vaadin.ui.HorizontalLayout;
import com.vaadin.ui.OptionGroup;
import com.vaadin.ui.Panel;
import com.vaadin.ui.PopupDateField;
import com.vaadin.ui.SplitPanel;
import com.vaadin.ui.Table;
import com.vaadin.ui.VerticalLayout;
import com.vaadin.ui.Button.ClickEvent;

/**
 * 
 * @author Xavier Junyent <xavierjunyent@yahoo.es>
 *
 */
@SuppressWarnings("serial")
public class JustifyMissesLayout extends VerticalLayout{
	private AttendancemanagerApplication app;
	private static ResourceBundle locale = AttendancemanagerApplication.locale;
	private static DatabaseStorage storage = AttendancemanagerApplication.storage;
	private static DateFormat df = AttendancemanagerApplication.df;
	
	private boolean adminmode = false;
	private boolean loading = false;
	
	private Table missTable;
	private Table excusedTable;
	
	private ComboBox groupsCombo;
	private ComboBox studentsCombo;
	private ComboBox termsCombo;
	
	private PopupDateField startDate;
	private PopupDateField endDate;
	
	private Button reloadCombosButton;
	private Button missButton;
	private Button selectallMissButton;
	private Button deselectallMissButton;
	private Button excusedButton;
	private Button selectallExcusedButton;
	private Button deselectallExcusedButton;
	
    private List<String> options;
	private OptionGroup option;
	
	public JustifyMissesLayout() {
		
	}
	
	public JustifyMissesLayout(boolean admin) {
		adminmode = admin;
	}
	
	public boolean isLoading() {
		return loading;
	}

	public void setLoading(boolean loading) {
		this.loading = loading;
	}

	private void initLayout(){
		//BUTTONS
		
		missButton= new Button();
		missButton.setIcon(new ThemeResource("img/24-em-down.png"));
		missButton.setEnabled(false);
		missButton.setWidth("100%");
		missButton.setStyleName("small");
		missButton.setDescription(locale.getString("JUSTIFIEDMISSESLAYOUT_TOEXCUSED_DESCRIPTION"));
		
		excusedButton= new Button();
		excusedButton.setIcon(new ThemeResource("img/24-em-up.png"));
		excusedButton.setEnabled(false);
		excusedButton.setWidth("100%");
		excusedButton.setStyleName("small");
		excusedButton.setDescription(locale.getString("JUSTIFIEDMISSESLAYOUT_TOMISS_DESCRIPTION"));
		
		selectallMissButton=new Button(locale.getString("JUSTIFIEDMISSESLAYOUT_SELECTALLBUTTON_CAPTION"));
		selectallMissButton.setStyleName("small");
		selectallMissButton.setEnabled(false);
		
		deselectallMissButton=new Button(locale.getString("JUSTIFIEDMISSESLAYOUT_DESELECTALLBUTTON_CAPTION"));
		deselectallMissButton.setStyleName("small");
		deselectallMissButton.setEnabled(false);
		
		selectallExcusedButton=new Button(locale.getString("JUSTIFIEDMISSESLAYOUT_SELECTALLBUTTON_CAPTION"));
		selectallExcusedButton.setStyleName("small");
		selectallExcusedButton.setEnabled(false);
		
		deselectallExcusedButton=new Button(locale.getString("JUSTIFIEDMISSESLAYOUT_DESELECTALLBUTTON_CAPTION"));
		deselectallExcusedButton.setStyleName("small");
		deselectallExcusedButton.setEnabled(false);
		
		reloadCombosButton= new Button(locale.getString("JUSTIFIEDMISSESLAYOUT_RELOADCOMBOSBUTTON_CAPTION"));
		//reloadCombosButton.setIcon(new ThemeResource("img/24-em-down.png"));
		reloadCombosButton.setEnabled(true);
		reloadCombosButton.setWidth("100%");
		reloadCombosButton.setStyleName("small");
		
		options = Arrays.asList(new String[] {locale.getString("JUSTIFIEDMISSESLAYOUT_OPTIONBYTERM_CAPTION"), locale.getString("JUSTIFIEDMISSESLAYOUT_OPTIONBYDATE_CAPTION")});
		option = new OptionGroup(locale.getString("JUSTIFIEDMISSESLAYOUT_OPTION_CAPTION"), options);
		option.setImmediate(true);
		
		
		// LAYOUTS (students, groups)
		GridLayout combosGridLayout =new  GridLayout();
		combosGridLayout.setSpacing(true);
		combosGridLayout.setMargin(false);
		combosGridLayout.setColumns(12);
		combosGridLayout.setRows(2);
		
		
		// LAYOUTS (students, groups)
		VerticalLayout tablesLayout =new  VerticalLayout();
		tablesLayout.setSpacing(true);
		tablesLayout.setMargin(false);
		
		//LAYOUT BUTTON ADD JUSTIFIED
		HorizontalLayout missButtonLayout = new HorizontalLayout();
		missButtonLayout.addComponent(selectallMissButton);
		missButtonLayout.addComponent(deselectallMissButton);
		missButtonLayout.addComponent(missButton);
		missButtonLayout.setSpacing(true);
		missButtonLayout.setMargin(false);
		missButtonLayout.setWidth("100%");
		missButtonLayout.setExpandRatio(missButton, 1);
		missButtonLayout.setComponentAlignment(missButton, Alignment.MIDDLE_CENTER);
		
		//LAYOUT BUTTON REMOVE JUSTIFIED
		HorizontalLayout excusedButtonLayout = new HorizontalLayout();
		excusedButtonLayout.addComponent(selectallExcusedButton);
		excusedButtonLayout.addComponent(deselectallExcusedButton);
		excusedButtonLayout.addComponent(excusedButton);
		excusedButtonLayout.setSpacing(true);
		excusedButtonLayout.setMargin(false);
		excusedButtonLayout.setWidth("100%");
		excusedButtonLayout.setExpandRatio(excusedButton, 1);
		excusedButtonLayout.setComponentAlignment(excusedButton, Alignment.MIDDLE_CENTER);
		
		//COMBOS (groups,students)
		groupsCombo = new ComboBox(locale.getString("GROUPMANAGERLAYOUT_GROUPSPANEL_CAPTION"));
		groupsCombo.setNullSelectionAllowed(false);
		groupsCombo.setImmediate(true);
		groupsCombo.setRequired(false);
		groupsCombo.setFilteringMode(ComboBox.FILTERINGMODE_CONTAINS);
		groupsCombo.setReadThrough(true);	
		groupsCombo.setWidth("100%");
		
		studentsCombo = new ComboBox(locale.getString("STUDENTSMANAGERLAYOUT_STUDENTSPANEL_CAPTION"));
		studentsCombo.setNullSelectionAllowed(false);
		studentsCombo.setImmediate(true);
		studentsCombo.setRequired(true);
		studentsCombo.setFilteringMode(ComboBox.FILTERINGMODE_CONTAINS);
		studentsCombo.setReadThrough(true);
		studentsCombo.setWidth("100%");
		
		termsCombo = new ComboBox(locale.getString("TERMMANAGERLAYOUT_TERMSPANEL_CAPTION"));
		termsCombo.setNullSelectionAllowed(false);
		termsCombo.setImmediate(true);
		termsCombo.setRequired(true);
		termsCombo.setFilteringMode(ComboBox.FILTERINGMODE_CONTAINS);
		termsCombo.setReadThrough(true);
		termsCombo.setWidth("100%");
		
		//POPUPS DATE
		startDate= new PopupDateField (locale.getString("REPORTSLAYOUT_STARTDATE_CAPTION"));
		startDate.setValue(new java.util.Date());
		startDate.setResolution(PopupDateField.RESOLUTION_DAY);
		startDate.setImmediate(true);

		endDate= new PopupDateField (locale.getString("REPORTSLAYOUT_ENDDATE_CAPTION"));
		endDate.setValue(new java.util.Date());
		endDate.setResolution(PopupDateField.RESOLUTION_DAY);
		endDate.setImmediate(true);
		
		combosGridLayout.addComponent(option);
		combosGridLayout.addComponent(groupsCombo);
		combosGridLayout.addComponent(studentsCombo,1,1);
		combosGridLayout.addComponent(termsCombo);
		combosGridLayout.addComponent(startDate,3,0);
		combosGridLayout.addComponent(endDate,3,1);
		combosGridLayout.addComponent(reloadCombosButton,0,1);
		combosGridLayout.setComponentAlignment(reloadCombosButton, Alignment.BOTTOM_RIGHT);
		
		//TABLES (JUSTIFY MISS)
		missTable = new Table(){
		@Override
		            protected String formatPropertyValue(Object rowId, Object colId,
		                    Property property) {
		                Object v = property.getValue();
		                if (v instanceof Date) {
		                    return df.format(v);
		                }
		                return super.formatPropertyValue(rowId, colId, property);
		            }
		
		};
		missTable.setImmediate(true);
		missTable.setSelectable(true);
		missTable.setMultiSelect(true);
		missTable.setColumnReorderingAllowed(true);
		missTable.setColumnCollapsingAllowed(true);
		missTable.setStyleName("striped");
		missTable.setSizeFull();

		//TABLES (NO JUSTIFY MISS)
		excusedTable = new Table(){
			@Override
            protected String formatPropertyValue(Object rowId, Object colId,
                    Property property) {
                Object v = property.getValue();
                if (v instanceof Date) {
                    return df.format(v);
                }
                return super.formatPropertyValue(rowId, colId, property);
            }

		};
		excusedTable.setImmediate(true);
		excusedTable.setSelectable(true);
		excusedTable.setMultiSelect(true);
		excusedTable.setColumnReorderingAllowed(true);
		excusedTable.setColumnCollapsingAllowed(true);
		excusedTable.setStyleName("striped");
		excusedTable.setSizeFull();
		
		
		//PANEL (WITH COMBOS SEARCH)
		Panel searchPanel = new Panel(locale.getString("JUSTIFIEDMISSESLAYOUT_SEARCHPANEL_CAPTION"));
		//searchPanel.setSizeFull(); // Corregir problema con el layout
		searchPanel.addComponent(combosGridLayout);
		searchPanel.setStyleName("bubble");
		searchPanel.setIcon(new ThemeResource("img/flag_blue.png"));
		searchPanel.getContent().setSizeFull();
		
		
		//PANELS (WITH TABLE JUSTIFY MISS)
		
		//Buttons add,selectall,deselectall  must be on top
		Panel missPanel = new Panel(locale.getString("JUSTIFIEDMISSESLAYOUT_MISSPANEL_CAPTION"));
		missPanel.addComponent(missTable);
		missPanel.addComponent(missButtonLayout);
		missPanel.setStyleName("bubble");
		missPanel.setIcon(new ThemeResource("img/flag_red.png"));
		missPanel.setSizeFull();
		missPanel.getContent().setSizeFull();
		


		//PANELS (WITH TABLE NO JUSTIFY MISS)
		Panel excusedPanel = new Panel(locale.getString("JUSTIFIEDMISSESLAYOUT_EXCUSEDPANEL_CAPTION"));
		excusedPanel.addComponent(excusedTable);
		excusedPanel.addComponent(excusedButtonLayout);
		excusedPanel.setStyleName("bubble");
		excusedPanel.setIcon(new ThemeResource("img/flag_green.png"));
		excusedPanel.setSizeFull();
		excusedPanel.getContent().setSizeFull();
	
		
		VerticalLayout excusedpanelLayout = new VerticalLayout();
		excusedpanelLayout.addComponent(excusedButtonLayout);
		excusedpanelLayout.addComponent(excusedPanel);
		excusedpanelLayout.setSizeFull();
		excusedpanelLayout.setExpandRatio(excusedPanel, 1);
		
		//SPLIT PANEL WITH PANELS (MISSES)
		
		SplitPanel missesSplitPanel = new SplitPanel(SplitPanel.ORIENTATION_VERTICAL);
		missesSplitPanel.addComponent(missPanel);
		missesSplitPanel.addComponent(excusedpanelLayout);

		missesSplitPanel.setSizeFull();
		
		VerticalLayout vl =  (VerticalLayout) excusedPanel.getContent();
		vl.setMargin(false);
		vl.setExpandRatio(excusedTable, 1.0f);
		vl.setSpacing(false);

		
		vl=(VerticalLayout) missPanel.getContent();
		vl.setMargin(false);
		vl.setExpandRatio(missTable, 1.0f);
		vl.setSpacing(false);
		

		
		//ADD COMPONENTS
		addComponent(searchPanel);
		addComponent(missesSplitPanel);
		
		//PROPERTIES LAYOUT
		
		setSizeFull();
		setMargin(false, true, false, true);
		
		setExpandRatio(missesSplitPanel, 1.0f);
		setSpacing(true);
	}
	
	public void attach(){
		
		app = (AttendancemanagerApplication)getApplication();
		
		initLayout();
		
		missTable.addContainerProperty(DatabaseStorage.DATE_PROPERTY, java.util.Date.class, "");
		missTable.addContainerProperty(DatabaseStorage.START_PROPERTY, String.class, "");
		missTable.addContainerProperty(DatabaseStorage.END_PROPERTY, String.class, "");
		missTable.addContainerProperty(DatabaseStorage.SURNAME1_PROPERTY,String.class, "");
		missTable.addContainerProperty(DatabaseStorage.SURNAME2_PROPERTY,String.class, "");
		missTable.addContainerProperty(DatabaseStorage.NAME_PROPERTY, String.class, "");
		missTable.addContainerProperty(DatabaseStorage.GROUPNAME_PROPERTY, String.class, "");
		missTable.addContainerProperty(DatabaseStorage.SUBJECTNAME_PROPERTY, String.class, "");
		missTable.addContainerProperty(DatabaseStorage.TEACHERNAME_PROPERTY, String.class, "");
		missTable.setColumnHeaders(new String[] {
				locale.getString("JUSTIFIEDMISSESLAYOUT_TABLE_DATECOLUMN"),
				locale.getString("TERMMANAGERLAYOUT_TERMSTABLE_STARTCOLUMN"),
				locale.getString("TERMMANAGERLAYOUT_TERMSTABLE_ENDCOLUMN"),
				locale.getString("GROUPMANAGERLAYOUT_STUDENTGROUPTABLE_SURNAME1"),
				locale.getString("GROUPMANAGERLAYOUT_STUDENTGROUPTABLE_SURNAME2"),
				locale.getString("GROUPMANAGERLAYOUT_GROUPSTABLE_NAME"),
				locale.getString("CLASSBLOCKMANAGERLAYOUT_CLASSBLOCKTABLE_GROUPNAME"),
				locale.getString("CLASSBLOCKMANAGERLAYOUT_CLASSBLOCKTABLE_SUBJECTNAME"),
				locale.getString("CLASSBLOCKMANAGERLAYOUT_CLASSBLOCKTABLE_TEACHERNAME")
		});
		missTable.addListener(new Property.ValueChangeListener() {
			@Override
			public void valueChange(ValueChangeEvent event) {

				Set<?> value = (Set<?>) event.getProperty().getValue();
				if (null == value || value.size() == 0) {
					missButton.setEnabled(false);
				} else {
					missButton.setEnabled(true);
				}				
			}
		});
		
		excusedTable.addContainerProperty(DatabaseStorage.DATE_PROPERTY, java.util.Date.class, "");
		excusedTable.addContainerProperty(DatabaseStorage.START_PROPERTY, String.class, "");
		excusedTable.addContainerProperty(DatabaseStorage.END_PROPERTY, String.class, "");
		excusedTable.addContainerProperty(DatabaseStorage.SURNAME1_PROPERTY, String.class, "");
		excusedTable.addContainerProperty(DatabaseStorage.SURNAME2_PROPERTY, String.class, "");
		excusedTable.addContainerProperty(DatabaseStorage.NAME_PROPERTY, String.class, "");
		excusedTable.addContainerProperty(DatabaseStorage.GROUPNAME_PROPERTY, String.class, "");
		excusedTable.addContainerProperty(DatabaseStorage.SUBJECTNAME_PROPERTY, String.class, "");
		excusedTable.addContainerProperty(DatabaseStorage.TEACHERNAME_PROPERTY, String.class, "");
		excusedTable.setColumnHeaders(new String[] {
				locale.getString("JUSTIFIEDMISSESLAYOUT_TABLE_DATECOLUMN"),
				locale.getString("TERMMANAGERLAYOUT_TERMSTABLE_STARTCOLUMN"),
				locale.getString("TERMMANAGERLAYOUT_TERMSTABLE_ENDCOLUMN"),
				locale.getString("GROUPMANAGERLAYOUT_STUDENTGROUPTABLE_SURNAME1"),
				locale.getString("GROUPMANAGERLAYOUT_STUDENTGROUPTABLE_SURNAME2"),
				locale.getString("GROUPMANAGERLAYOUT_GROUPSTABLE_NAME"),
				locale.getString("CLASSBLOCKMANAGERLAYOUT_CLASSBLOCKTABLE_GROUPNAME"),
				locale.getString("CLASSBLOCKMANAGERLAYOUT_CLASSBLOCKTABLE_SUBJECTNAME"),
				locale.getString("CLASSBLOCKMANAGERLAYOUT_CLASSBLOCKTABLE_TEACHERNAME")
		});


		excusedTable.addListener(new Property.ValueChangeListener() {
			@Override
			public void valueChange(ValueChangeEvent event) {

				Set<?> value = (Set<?>) event.getProperty().getValue();
				if (null == value || value.size() == 0) {
					excusedButton.setEnabled(false);

				} else {
					excusedButton.setEnabled(true);
				}				
			}
		});
		
		excusedButton.addListener(new Button.ClickListener() {
			@Override
			public void buttonClick(ClickEvent event) {

				Set<?> misses = (Set<?>) excusedTable.getValue();;
				if (null != misses && misses.size() > 0) {
					storage.updateExcusedtoMiss(misses);
					refreshMiss();
				}
			}
		});
		
		missButton.addListener(new Button.ClickListener() {
			@Override
			public void buttonClick(ClickEvent event) {
				Set<?> misses = (Set<?>) missTable.getValue();;
				if (null != misses && misses.size() > 0) {
					storage.updateMisstoExcused(misses);
					refreshMiss();
				}
			}
		});
		
		reloadCombosButton.addListener(new Button.ClickListener() {
			@Override
			public void buttonClick(ClickEvent event) {
				refreshLayout();
			}
		});
				
		groupsCombo.addListener(new Property.ValueChangeListener() {
			@Override
			public void valueChange(ValueChangeEvent event) {
				if(loading) return;
				
				Integer idgroupselected = getSelectedGroup();
				refreshStudentsCombo(idgroupselected);
				
				refreshMiss();
			}
		});
		
		studentsCombo.addListener(new Property.ValueChangeListener() {
			@Override
			public void valueChange(ValueChangeEvent event) {
				if(loading) return;
				
				refreshMiss();
			}
		});
		
		termsCombo.addListener(new Property.ValueChangeListener() {
			@Override
			public void valueChange(ValueChangeEvent event) {
				if(loading) return;
				refreshMiss();
			}
		});
		
		selectallMissButton.addListener(new Button.ClickListener() {
			@Override
			public void buttonClick(ClickEvent event) {
				for(Object value :missTable.getItemIds()){
					missTable.select(value);
				}
			}
		});
		
		selectallExcusedButton.addListener(new Button.ClickListener() {
			
			@Override
			public void buttonClick(ClickEvent event) {

				for(Object value :excusedTable.getItemIds()){
					excusedTable.select(value);
				}
			}
		});
		
		deselectallMissButton.addListener(new Button.ClickListener() {
			
			@Override
			public void buttonClick(ClickEvent event) {
					missTable.setValue(null);
			}
		});
		
		deselectallExcusedButton.addListener(new Button.ClickListener() {
			
		@Override
		public void buttonClick(ClickEvent event) {
					excusedTable.setValue(null);
			}
		});

		startDate.addListener(new Property.ValueChangeListener() {
			@Override
			public void valueChange(ValueChangeEvent event) {
				if(loading) return;
				refreshMiss();
			}
		});
		
		endDate.addListener(new Property.ValueChangeListener() {
			@Override
			public void valueChange(ValueChangeEvent event) {
				if(loading) return;
				refreshMiss();
			}
		});
		
		option.addListener(new Property.ValueChangeListener() {
			@Override
			public void valueChange(ValueChangeEvent event) {
				if(loading) return;

				if (event.getProperty().getValue().equals(locale.getString("JUSTIFIEDMISSESLAYOUT_OPTIONBYTERM_CAPTION"))){
					setEnabledTerm(true);
				}else{
					setEnabledTerm(false);
				}
				refreshMiss();
			}
		});
		
		// Con modoadmin=false no carga los datos hasta que no se pulsa
		// el botón 'por defecto'. Por eso refrescamos aquí.
		refreshLayout();
	}

	public void refreshLayout(){
		Logger.log("JustifyMissesLayout.refreshLayout() admin: " + adminmode);
		
		// Realizamos la carga inicial de datos
		setLoading(true);
		
		refreshDates();
		refreshTermsCombo();
		setEnabledTerm(true);
		refreshOptions();
		refreshGroupsCombo();
		int idgroupselected = getSelectedGroup();
		refreshStudentsCombo(idgroupselected);
		
		refreshMiss();
		
		// Hemos acabado de cargar datos
		setLoading(false);
	}
	
	private void setEnabledTerm(boolean value){
		termsCombo.setEnabled(value);
		startDate.setEnabled(! value);
		endDate.setEnabled(! value);
	}
	
	private void refreshMiss(){
		Logger.log("JustifyMissesLayout.refreshMiss()");
		
		Date start =(Date) (option.getValue()==locale.getString("JUSTIFIEDMISSESLAYOUT_OPTIONBYTERM_CAPTION") ? termsCombo.getContainerDataSource().getContainerProperty(termsCombo.getValue(), "start").getValue() : startDate.getValue());
		Date end =(Date) (option.getValue()==locale.getString("JUSTIFIEDMISSESLAYOUT_OPTIONBYTERM_CAPTION") ? termsCombo.getContainerDataSource().getContainerProperty(termsCombo.getValue(), "end").getValue() : endDate.getValue());	

		int studentid = getSelectedStudent();
		int groupid = getSelectedGroup();
		
		//QueryContainer excusedquerycontainer=storage.getMissQueryContainer(Miss.EXCUSED_MISS, getSelectedStudent(), getSelectedGroup(), app.user, start, end);
		
		QueryContainer excusedquerycontainer;
		if(studentid!=0) {
			// Si hay un estudiante seleccionado, recuperamos sus faltas
			excusedquerycontainer = storage.getMissQueryContainerByStudent(Miss.EXCUSED_MISS, start, end, studentid);
		} else if(groupid!=0) {
			// Si hay un grupo seleccionado, recuperamos las faltas de sus alumnos
			excusedquerycontainer = storage.getMissQueryContainerByGroup(Miss.EXCUSED_MISS, start, end, groupid);
		} else if(adminmode) {
			// Si estamos en modo admin, recuperamos las faltas de todos los alumnos
			excusedquerycontainer = storage.getMissQueryContainerAdminMode(Miss.EXCUSED_MISS, start, end);
		} else {
			// Si todo lo anterior es falso, recuperamos las faltas de los alumnos de
			// cualquiera de los grupos de los que el usuario es tutor
			excusedquerycontainer = storage.getMissQueryContainerByTeacher(Miss.EXCUSED_MISS, start, end, app.user.getID());
		}
		storage.refreshTable(excusedquerycontainer, excusedTable);

		if(excusedquerycontainer.size()>0){ 
			selectallExcusedButton.setEnabled(true);
			deselectallExcusedButton.setEnabled(true);
		}else{
			selectallExcusedButton.setEnabled(false);
			deselectallExcusedButton.setEnabled(false);
		}
		

		//QueryContainer missquerycontainer=storage.getMissQueryContainer(Miss.MISS,getSelectedStudent(),getSelectedGroup(),app.user,start,end);

		QueryContainer missquerycontainer;
		if(studentid!=0) {
			missquerycontainer = storage.getMissQueryContainerByStudent(Miss.MISS, start, end, studentid);
		} else if(groupid!=0) {
			missquerycontainer = storage.getMissQueryContainerByGroup(Miss.MISS, start, end, groupid);
		} else if(adminmode) {
			missquerycontainer = storage.getMissQueryContainerAdminMode(Miss.MISS, start, end);
		} else {
			missquerycontainer = storage.getMissQueryContainerByTeacher(Miss.MISS, start, end, app.user.getID());
		}
		storage.refreshTable(missquerycontainer, missTable);
		
		if(missquerycontainer.size()>0){ 
			selectallMissButton.setEnabled(true);
			deselectallMissButton.setEnabled(true);
		}else{
			selectallMissButton.setEnabled(false);
			deselectallMissButton.setEnabled(false);
		}
	}
	
	private int getSelectedGroup(){
		return (Integer) groupsCombo.getValue();
	}
	
	private int getSelectedStudent(){
		return (Integer) studentsCombo.getValue();
	} 
	
	private void refreshDates(){
		Logger.log("JustifyMissesLayout.refreshDates()");
		Calendar c1 = Calendar.getInstance();
		
		c1.set(Calendar.DATE, 1);
		startDate.setValue(c1.getTime());
		
		c1.add(Calendar.MONTH, 1);
		c1.add(Calendar.DATE, -1);
		endDate.setValue(c1.getTime());
	}
	
	private void refreshOptions(){
		Logger.log("JustifyMissesLayout.refreshOptions()");
		option.setValue(locale.getString("JUSTIFIEDMISSESLAYOUT_OPTIONBYTERM_CAPTION"));
	}
	
	private void refreshGroupsCombo(){
		Logger.log("JustifyMissesLayout.refreshGroupsCombo()");

		Collection<Group> collection;
		if(adminmode) {
			collection = storage.allGroups();
		} else {
			collection = storage.allGroups(app.user);
		}
		
		IndexedContainer ic=new IndexedContainer();
		ic.addContainerProperty("namegroup", String.class, null);
		Iterator<Group> it =collection.iterator();
		Item itemGroup=ic.addItem(0);
		itemGroup.getItemProperty("namegroup").setValue(locale.getString("REPORTSLAYOUT_LOADTERMS_ALLGROUPS"));
		while (it.hasNext()){
			Group group=it.next();
			itemGroup=ic.addItem(group.getID());
			itemGroup.getItemProperty("namegroup").setValue(group.getName());
		}
		groupsCombo.setContainerDataSource(ic);
		groupsCombo.setItemCaptionPropertyId("namegroup");
		groupsCombo.setValue(0);
	}
	
	private void refreshStudentsCombo(int idgroup){
		Logger.log("JustifyMissesLayout.refreshStudentsCombo()");
		
		/**
		 * Si adminmode == true TODOS LOS GRUPOS es literal,
		 * en caso contrario TODOS LOS GRUPOS significa todos
		 * los grupos de los que yo soy tutor.
		 */
		Collection<Student> collection;
		if(adminmode) {
			collection = storage.getStudentsCollection(idgroup);
		} else {
			collection = storage.getStudentsCollection(idgroup, app.user.getID());
		}
		
		IndexedContainer ic=new IndexedContainer();
		ic.addContainerProperty("fullname", String.class, null);
		Iterator<Student> it =collection.iterator();
		Item itemStudent=ic.addItem(0);
		itemStudent.getItemProperty("fullname").setValue(locale.getString("JUSTIFIEDMISSESLAYOUT_REFRESHSTUDENTSCOMBOBOX_ALLSTUDENTS"));
		while (it.hasNext()){
			Student student=it.next();
			itemStudent=ic.addItem(student.getID());
			itemStudent.getItemProperty("fullname").setValue( student.getFullName() );
		}
		studentsCombo.setContainerDataSource(ic);
		studentsCombo.setItemCaptionPropertyId("fullname");
		studentsCombo.setValue(0);	
	}
	
	private void refreshTermsCombo(){
		Logger.log("JustifyMissesLayout.refreshTermsCombo()");

		Collection<Term> collection = storage.allTerms();
//		System.out.println("collection: " + collection);
		
		IndexedContainer ic = new IndexedContainer();
		ic.addContainerProperty("term", String.class, null);
		ic.addContainerProperty("start", java.util.Date.class, null);
		ic.addContainerProperty("end", java.util.Date.class, null);
//		System.out.println("ic: " + ic);
		
		Item itemTerm = ic.addItem(0);
		itemTerm.getItemProperty("term").setValue(locale.getString("REPORTSLAYOUT_LOADTERMS_ALLTERMS"));
		itemTerm.getItemProperty("start").setValue(Constants.MIN_DATE);
		itemTerm.getItemProperty("end").setValue(Constants.MAX_DATE);
//		System.out.println("itemTerm: " + itemTerm);
		
		Iterator<Term> it = collection.iterator();
		while (it.hasNext()){
			Term term = it.next();
//			System.out.println("Term: " + term);
			itemTerm = ic.addItem(term.getID());
			itemTerm.getItemProperty("term").setValue(term.getName());
			itemTerm.getItemProperty("start").setValue(term.getStart());
			itemTerm.getItemProperty("end").setValue(term.getEnd());
//			System.out.println("itemTerm: " + itemTerm);
		}
		
//		System.out.println("ic: " + ic);
		termsCombo.setContainerDataSource(ic);
		termsCombo.setItemCaptionPropertyId("term");
		termsCombo.setValue(0);
//		System.out.println("termsCombo: " + termsCombo);
	}

}