/*
    AttendanceManager
    Copyright (C) 2010  Victor Carceler

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
 */

package org.iespuigcastellar.attendancemanager.screenlayouts;


import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Iterator;
import java.util.ResourceBundle;
import java.util.Set;


import org.iespuigcastellar.attendancemanager.AttendancemanagerApplication;


import org.iespuigcastellar.attendancemanager.core.DatabaseStorage;
import org.iespuigcastellar.attendancemanager.core.Group;
import org.iespuigcastellar.attendancemanager.core.Mailer;
import org.iespuigcastellar.attendancemanager.core.SMS;

import com.vaadin.data.Container;
import com.vaadin.data.Item;
import com.vaadin.data.Property;
import com.vaadin.data.Validator;
import com.vaadin.data.Property.ValueChangeEvent;
import com.vaadin.data.Property.ValueChangeListener;
import com.vaadin.data.Validator.InvalidValueException;
import com.vaadin.data.util.BeanItem;
import com.vaadin.data.util.IndexedContainer;
import com.vaadin.data.util.QueryContainer;
import com.vaadin.data.validator.AbstractStringValidator;
import com.vaadin.data.validator.AbstractValidator;
import com.vaadin.data.validator.IntegerValidator;
import com.vaadin.data.validator.StringLengthValidator;
import com.vaadin.terminal.ThemeResource;
import com.vaadin.ui.AbstractField;
import com.vaadin.ui.AbstractSelect;
import com.vaadin.ui.Alignment;
import com.vaadin.ui.Button;
import com.vaadin.ui.CheckBox;
import com.vaadin.ui.ComboBox;
import com.vaadin.ui.Component;
import com.vaadin.ui.CustomComponent;
import com.vaadin.ui.CustomField;
import com.vaadin.ui.DefaultFieldFactory;
import com.vaadin.ui.Field;
import com.vaadin.ui.Form;
import com.vaadin.ui.FormFieldFactory;
import com.vaadin.ui.GridLayout;
import com.vaadin.ui.HorizontalLayout;
import com.vaadin.ui.Label;
import com.vaadin.ui.ListSelect;
import com.vaadin.ui.Panel;
import com.vaadin.ui.TextField;
import com.vaadin.ui.TwinColSelect;
import com.vaadin.ui.VerticalLayout;
import com.vaadin.ui.Window;
import com.vaadin.ui.Button.ClickEvent;
import com.vaadin.ui.Button.ClickListener;
import com.vaadin.ui.Window.Notification;

/**
 * 
 * @author Xavier Junyent <xavierjunyent@yahoo.es>
 *
 */
@SuppressWarnings("serial")
public class MessageLayout extends VerticalLayout{
	/**
	 * 
	 */
	private AttendancemanagerApplication app;
	private ResourceBundle rb = AttendancemanagerApplication.rb;
	private ResourceBundle locale = AttendancemanagerApplication.locale;
	private DatabaseStorage storage = AttendancemanagerApplication.storage;
	
	private boolean adminmode = false;
	
	private final Form messageform = new Form();
	private final MessageInfo messageInfo = new MessageInfo();
	private Label informationLabel;
	private TextField fromtext;
	private TextField subjecttext;
	private TextField messagetext;
	
	
	private Button sendbutton;
	private Button discardbutton;
	
	private Panel rootPanel;
	
	public MessageLayout() {
		
	}
	
	public MessageLayout(boolean admin) {
		adminmode = admin;
	}
	
	public void initlayout(){

		//Panel
		 rootPanel=new Panel(locale.getString("MESSAGELAYOUT_ROOTPANEL_CAPTION"));
		rootPanel.setWidth("40%");
		rootPanel.setStyleName("bubble");
		rootPanel.setIcon(new ThemeResource("img/email.png"));
		
		informationLabel= new Label("",Label.CONTENT_XML);
		informationLabel.setCaption(locale.getString("MESSAGELAYOUT_INFORMATIONLABEL_CAPTION") );
		informationLabel.setIcon(new ThemeResource("img/information.png"));
		//Form
		BeanItem<MessageInfo> messagebean = new BeanItem<MessageInfo>(messageInfo);
		messageform.setItemDataSource(messagebean);
		messageform.setFormFieldFactory(new MyFormFieldFactory());
        // Determines which properties are shown, and in which order:
		messageform.setVisibleItemProperties(Arrays.asList(new String[] {
                "sendoptions", "from", "to","sendtome","subject","message"}));
		messageform.setValidationVisible(true);
		messageform.setImmediate(true);
		
		messageform.getField("sendtome").setValue(true);
		messageform.getField("sendtome").setCaption(locale.getString("MESSAGELAYOUT_SENDMECHECKBOX_CAPTION"));
		
		fromtext = (TextField)messageform.getField("from");
		fromtext.setWidth("100%");
		fromtext.setValue(rb.getString("mailer_from"));
		fromtext.setReadOnly(true);
		messageform.getField("from").setRequired(true);
		messageform.getField("from").setCaption(locale.getString("MESSAGELAYOUT_FROMTEXT_CAPTION"));
		messageform.getField("from").setRequiredError(locale.getString("MESSAGELAYOUT_FROMTEXT_REQUIREDERROR"));
		
		subjecttext = (TextField)messageform.getField("subject");
		subjecttext.setWidth("100%");
		subjecttext.setRequired(false);
		subjecttext.setCaption(locale.getString("MESSAGELAYOUT_SUBJECTFIELD_CAPTION"));
		subjecttext.setRequiredError(locale.getString("MESSAGELAYOUT_SUBJECTMESSAGE_REQUIREDERROR"));
		
		messagetext = (TextField)messageform.getField("message");
		messagetext.setWidth("100%");
		messagetext.setRows(5);
		messagetext.setImmediate(true);
		messagetext.setCaption(locale.getString("MESSAGELAYOUT_MESSAGETEXT_CAPTION"));
		
		messageform.getField("to").setCaption(locale.getString("MESSAGELAYOUT_TOTEXT_CAPTION"));
		messageform.getField("to").setRequiredError(locale.getString("MESSAGELAYOUT_TOTEXT_REQUIREDERROR"));
	
	
		
		
		//tolist.setRequiredError(locale.getString("MESSAGELAYOUT_TOTEXT_REQUIREDERROR"));
		sendbutton = new Button(locale.getString("MESSAGELAYOUT_SENDBUTTON_CAPTION"), this, "commit");
		sendbutton.setIcon(new ThemeResource("img/accept.png"));
		sendbutton.setStyleName("default");
		
		discardbutton = new Button(locale.getString("MESSAGELAYOUT_DISCARDBUTTON_CAPTION"),this,"discard");
		discardbutton.setIcon(new ThemeResource("img/cancel.png"));
		discardbutton.setStyleName("default");
		
		HorizontalLayout footerlayout=new HorizontalLayout(); 
		footerlayout.addComponent(sendbutton);
		footerlayout.addComponent(discardbutton);
		footerlayout.setSpacing(true);
		messageform.getFooter().addComponent(footerlayout);
		
		rootPanel.addComponent(informationLabel);
		rootPanel.addComponent(messageform);
		addComponent(rootPanel);

	}
	public void discard(){
		messageform.getField("message").setValue("");
		messageform.getField("subject").setValue("");

	}
	public void attach(){
		app = (AttendancemanagerApplication)getApplication();
		initlayout();
		messageform.getField("sendoptions").setValue(locale.getString("MESSAGELAYOUT_OPTIONSCOMBO_OPTION_EMAIL"));

	}
	
	public void commit(){
		try {
			messageform.commit();
		} catch (final InvalidValueException ive) {
			ive.printStackTrace();
			return;
		}
			//Mensaje enviado
			sendMessage();
	}
	/**
	 * Return String [] with email address added to send.
	 * 
	 */

	public String[] getMailsAddressToSend(){
		String to="";
		String addresses[]=null;
		ListSelectField f=(ListSelectField) messageform.getField("to");
		Container container=f.getContainerDataSource();
		Collection <?>collection=container.getItemIds();
		Iterator<?> it=collection.iterator();
		Integer id=null;
		if (isForSendTeacher()){to=app.user.getEmail();}
		while (it.hasNext()){
			id= (Integer) it.next();
			if ("".equals(to)){to=(String) container.getItem(id).getItemProperty("email").getValue();}
			else{to=to+";"+(String) container.getItem(id).getItemProperty("email").getValue();}
		}
		if (! to.equals("")){addresses=to.toString().split(";");}
		return addresses;
	}
	/**
	 * Return String [] with phones number added to send.
	 * 
	 */
	public String[] getPhonesNumbersToSend(){
		String to="";
		String phones[]=null;
		ListSelectField f=(ListSelectField) messageform.getField("to");
		Container container=f.getContainerDataSource();
		Collection <?>collection=container.getItemIds();
		Iterator<?> it=collection.iterator();
		Integer id=null;
		while (it.hasNext()){
			id= (Integer) it.next();
			to=to+";"+(String) container.getItem(id).getItemProperty("smsphone").getValue();
		}
		if (! to.equals("")){phones=to.toString().split(";");}
		return phones;
	}
	/**
	 * Send mail using the class Mailer.
	 * 
	 */
	private void sendMail(){
		
		Mailer newmail=new Mailer();
		
		newmail.setFrom((String)messageform.getField("from").getValue());
		newmail.setSubject((String)messageform.getField("subject").getValue());
		newmail.setBodyText((String)messageform.getField("message").getValue());
		
		String addresses[] = getMailsAddressToSend();
		String notification=null;
		for(int i=0; i<addresses.length; i++) {
			newmail.setTo(getMailsAddressToSend());
			newmail.sendMail();
			notification += "<p> -> " + newmail.getToElements() + "</p>";
		}
		
		getParent().getWindow().showNotification(
					locale.getString("REPORTSLAYOUT_SUCCESSMAIL_MESSAGE") + "<br/>" + notification,
					Notification.TYPE_TRAY_NOTIFICATION);
			
	}
	/**
	 * Send SMS.
	 * 
	 */
	public void sendSms(){
		SMS sms=new SMS();
		
		for (Object number:getPhonesNumbersToSend()){
			sms.sendSMS((String)number, (String)messageform.getField("message").getValue());

		}
		getParent().getWindow().showNotification(
				"Messages by sms sended",
				Notification.TYPE_TRAY_NOTIFICATION);
	}
	
	/**
	 * Send Message.
	 * 
	 */
	public void sendMessage(){
		if (isMessgeToSendEmail() & getEmailActivated()){
				sendMail();
		}
		if(isMessageToSendSms() & getSmsActivated()){		
			sendSms();
		}	
	}
	
	/**
	 * Check if the selected option is for send mail.
	 * Return boolean
	 * 
	 */
	private boolean isMessgeToSendEmail(){
		if (messageform.getField("sendoptions").getValue().equals(locale.getString("MESSAGELAYOUT_OPTIONSCOMBO_OPTION_EMAIL")) || messageform.getField("sendoptions").getValue().equals(locale.getString("MESSAGELAYOUT_OPTIONSCOMBO_OPTION_SMSEMAIL"))){
			return true;
		}
		return false;
	}
	/**
	 * Check if the selected option is for send sms.
	 * Return boolean
	 * 
	 */
	public boolean isMessageToSendSms(){
		if (messageform.getField("sendoptions").getValue().equals(locale.getString("MESSAGELAYOUT_OPTIONSCOMBO_OPTION_SMS")) || messageform.getField("sendoptions").getValue().equals(locale.getString("MESSAGELAYOUT_OPTIONSCOMBO_OPTION_SMSEMAIL"))){
			return true;
		}
		return false;
	}
	public boolean isForSendTeacher(){		
			return (Boolean)messageform.getField("sendtome").getValue();
	}
	public boolean getSmsActivated(){
		if (rb.getString("activated_sms").equals("1")){
			return true;
		}
		return false;
	}
	public boolean getEmailActivated(){
		if (rb.getString("activated_email").equals("1")){
			return true;
		}
		return false;
	}
	public class MessageInfo {


		String sendoptions;
		String to = "";
		String from = "";
		String subject = "";
		String message = "";
		boolean sendtome;

		public boolean isSendtome() {
			return sendtome;
		}
		public void setSendtome(boolean sendtome) {
			this.sendtome = sendtome;
		}
		public String getSendoptions() {
			return sendoptions;
		}
		public void setSendoptions(String sendoptions) {
			this.sendoptions = sendoptions;
		}
		public String getSubject() {
			return subject;
		}
		public void setSubject(String subject) {
			this.subject = subject;
		}
		public String getTo() {
			return to;
		}
		public void setTo(String to) {
			this.to = to;
		}
		public String getFrom() {
			return from;
		}
		public void setFrom(String from) {
			this.from = from;
		}
		public String getMessage() {
			return message;
		}
		public void setMessage(String message) {
			this.message = message;
		}
	}	
	@SuppressWarnings("serial")
	public  class ListSelectField extends CustomField implements Container.Viewer {
	     private HorizontalLayout layout = new HorizontalLayout();
	     private ListSelect field=new ListSelect();
	     private Button button = new Button();
	     private Button button2 = new Button();
	     private VerticalLayout vl=new VerticalLayout();
	     public ListSelectField() {
     		 //Field
        	field.setSizeUndefined();
        	field.setItemCaptionPropertyId("fullname");
        	field.setRows(4);
        	field.setNullSelectionAllowed(false);
        	field.setNewItemsAllowed(false);
        	field.setImmediate(true);
        	field.setRequired(true);
        	field.setMultiSelect(true);
        	field.addContainerProperty("fullname", String.class, null);
        	field.addContainerProperty("email", String.class, null);
        	field.addContainerProperty("smsphone", String.class, null);
    
    		 //Buttons
    		 button.setStyleName("icon-only");
    		 button.setIcon(new ThemeResource("img/book_open.png"));
    		 button.addListener(new Button.ClickListener() {
				
				@Override
				public void buttonClick(ClickEvent event) {
					getWindow().addWindow(new SearchWindow());
				}
			});
    		 
    		 button2.setStyleName("icon-only");
    		 button2.setIcon(new ThemeResource("img/cross.png"));
    		 button2.addListener(new Button.ClickListener() {
				
				@Override
				public void buttonClick(ClickEvent event) {
					Set <?>values=(Set<?>) field.getValue();
					for (Object v:values){
						field.removeItem(v);
					}
				}
			});
    		 
    		 //layouts
    		 layout.setSizeFull();
    		 vl.setSizeUndefined();
    		 vl.setSpacing(true);
    		 vl.setHeight(100, UNITS_PERCENTAGE);
    		 vl.addComponent(button);
    		 vl.addComponent(button2);
      		 layout.addComponent(field);
      		 layout.addComponent(vl);
      		 vl.setComponentAlignment(button2, Alignment.BOTTOM_CENTER);
      		 super.setWidth(100, UNITS_PERCENTAGE);;
      		 setWidth(100, UNITS_PERCENTAGE);;
      		 layout.setWidth(100, UNITS_PERCENTAGE);;
      		 field.setWidth(100, UNITS_PERCENTAGE);;
      		 field.setRows(5);
      		 layout.setExpandRatio(field, 1.0f);
      		 layout.setSpacing(true);
      		 setImmediate(true);
      		 setCompositionRoot(layout);
        }

        @Override
        public Class<?> getType() {
            return ListSelect.class;
        }

		@Override
		public Container getContainerDataSource() {
			return field.getContainerDataSource();
		}

		@Override
		public void setContainerDataSource(Container newDataSource) {
				field.setContainerDataSource(newDataSource);
		}

		public Collection getItemIds() {
			return field.getItemIds();
		}
		
	}

	public class MyFormFieldFactory extends DefaultFieldFactory implements FormFieldFactory {
		    private static final long serialVersionUID = 5845192054295782123L;
		    final ComboBox optionsCombo = new ComboBox(locale.getString("MESSAGELAYOUT_OPTIONSCOMBOBOX_CAPTION"));
		    private Field f;
		    private StringLengthValidator stringlenghtvalidator; 
		    public Field createField(Item item, final Object propertyId, Component uiContext) {
		        if ("to".equals(propertyId)) {
		        	f=new ListSelectField();
		        	f.setRequired(true);
		        }else if ("sendoptions".equals(propertyId)){
		        		optionsCombo.setRequiredError(locale.getString("MESSAGELAYOUT_OPTIONSCOMBOBOX_REQUIREDERROR"));
		        		optionsCombo.setRequired(true);
		        		optionsCombo.setImmediate(true);
		        		optionsCombo.setNullSelectionAllowed(false);
		        		optionsCombo.addItem(locale.getString("MESSAGELAYOUT_OPTIONSCOMBO_OPTION_EMAIL"));
		        		optionsCombo.addItem(locale.getString("MESSAGELAYOUT_OPTIONSCOMBO_OPTION_SMS"));
		        		optionsCombo.addItem(locale.getString("MESSAGELAYOUT_OPTIONSCOMBO_OPTION_SMSEMAIL"));
		        		optionsCombo.addListener(new Property.ValueChangeListener() {
		        			@Override
		        			public void valueChange(ValueChangeEvent event) {
		        				
		        				
		        				if (stringlenghtvalidator!=null){
		        					messageform.getField("message").removeValidator(stringlenghtvalidator);
		        				}
		        					stringlenghtvalidator=getLengthMessageValidator(optionsCombo.getValue().toString());
		        					messageform.getField("message").addValidator(stringlenghtvalidator);
		        				
		        				//Information Label
		        				setInformationLabel(optionsCombo.getValue().toString());
		        			}
		        		});
		        		
		      
		                return optionsCombo;
		            }
		        else{
		            f = super.createField(item, propertyId, uiContext);
		        }  
		        
		        f.setParent(((Form)uiContext).getLayout());
		        f.setWidth("100%");
		        return f;       
		    }
		    private void setInformationLabel(String combooption){
		    	if (combooption.toString().equals(locale.getString("MESSAGELAYOUT_OPTIONSCOMBO_OPTION_EMAIL"))){
					//messageform.getField("from").setValue(app.user.getEmail());
					if (getEmailActivated()) { 
						informationLabel.setValue(locale.getString("MESSAGELAYOUT_INFORMATIONLABEL_VALUEEMAIL"));
					}
					else{
						informationLabel.setValue(locale.getString("MESSAGELAYOUT_INFORLABEL_EMAILNOACTIVATED_BY_ADMIN"));
					}
				}else if (combooption.toString().equals(locale.getString("MESSAGELAYOUT_OPTIONSCOMBO_OPTION_SMS"))){
					//messageform.getField("from").setValue(app.user.getFullname());
					if (getSmsActivated()) {
						informationLabel.setValue(locale.getString("MESSAGELAYOUT_INFORMATIONLABEL_VALUESMS"));
					}else{
						informationLabel.setValue(locale.getString("MESSAGELAYOUT_INFORLABEL_SMSNOACTIVATED_BY_ADMIN"));
					}
				}else{;
					//messageform.getField("from").setValue( app.user.getEmail() +"<"+ app.user.getFullname()+">" );
					if (! getSmsActivated()){
						if (! getEmailActivated()){
							informationLabel.setValue(locale.getString("MESSAGELAYOUT_INFORLABEL_SMSEMAILNOACTIVATED_BY_ADMIN"));
						}else{
							informationLabel.setValue(locale.getString("MESSAGELAYOUT_INFORLABEL_SMSNOACTIVATED_BY_ADMIN"));
						}
					}else {
						if (! getEmailActivated()){
							informationLabel.setValue(locale.getString("MESSAGELAYOUT_INFORLABEL_EMAILNOACTIVATED_BY_ADMIN"));
						}else{
							informationLabel.setValue(locale.getString("MESSAGELAYOUT_INFORMATIONLABEL_VALUESMSEMAIL"));
						}
					}
					
				}
		    	
		    }
		    private StringLengthValidator getLengthMessageValidator(String combooption){
		    	Integer maxmessagelenght=0;
		    	if (optionsCombo.getValue().toString().equals(locale.getString("MESSAGELAYOUT_OPTIONSCOMBO_OPTION_EMAIL"))){
		    		maxmessagelenght=Integer.parseInt(rb.getString("mail_max_lenght"));
		    		return new StringLengthValidator(locale.getString("MESSAGELAYOUT_TOOLONG_MESSAGE_ERROR") +  String.valueOf(maxmessagelenght) ,1,200,false);
		    	} else{
		    		Integer sms_max_number=Integer.parseInt(rb.getString("sms_max_number"));
		    		
		    		if (sms_max_number==1){ 
		    			maxmessagelenght=70;
		    		}else{
		    			maxmessagelenght=sms_max_number*63;
		    		}
		    		return new StringLengthValidator(locale.getString("MESSAGELAYOUT_TOOLONG_MESSAGE_ERROR") + String.valueOf(maxmessagelenght)  ,1,maxmessagelenght,false);
		    		
		    	}
		    }

		
		}
	public class SearchWindow extends Window{
		/**
		 * Window for select contacts (Students);
		 *
		 * 
		 */
		private static final long serialVersionUID = 1L;
		private ComboBox groupscombo;
		private TwinColSelect twincol = new TwinColSelect(locale.getString("MESSAGELAYOUT_SEARCHWINDOW_TWINCOL_CAPTION"));
		private Button addbutton;
		private SearchWindow(){
			addbutton=new Button(locale.getString("MESSAGELAYOUT_SEARCHWINDOW_ADDBUTTON_CAPTION"),new Button.ClickListener() {
				
				/**
				 * 
				 */
				private static final long serialVersionUID = 1L;

				@SuppressWarnings("unchecked")
				@Override
				public void buttonClick(ClickEvent event) {

					
					ArrayList<?> list = new ArrayList();
					list.addAll((Collection) twincol.getValue());
					ListSelectField field=(ListSelectField) messageform.getField("to");
					Container container=field.getContainerDataSource();
					Iterator <?>it=list.iterator();
					int id;
					while (it.hasNext())
					{
						id=(Integer) it.next();
					
						try{
							Item item=container.addItem(twincol.getContainerDataSource().getContainerProperty(id, "idstudent").getValue());
							item.getItemProperty("fullname").setValue(twincol.getContainerDataSource().getContainerProperty(id, "fullname").getValue());
							item.getItemProperty("email").setValue(twincol.getContainerDataSource().getContainerProperty(id, "email").getValue());
							item.getItemProperty("smsphone").setValue(twincol.getContainerDataSource().getContainerProperty(id, "smsphone").getValue());
//							
						}catch(Exception e){
							//System.out.println("Item allready added.");
							e.printStackTrace();
						}
					}
				}
				
			});
			addbutton.setIcon(new ThemeResource("img/add.png"));
			
			groupscombo=getGroupsComboBox();
			VerticalLayout vertical=new VerticalLayout();
			vertical.setSizeFull();
			vertical.addComponent(groupscombo);
			vertical.addComponent(twincol);
			
			vertical.addComponent(addbutton);
			twincol.setWidth("500px");
			groupscombo.setImmediate(true);
			groupscombo.setSizeFull();
			
			VerticalLayout windowlayout = (VerticalLayout) getContent();
			windowlayout.setSpacing(true);
			windowlayout.setMargin(true);
			windowlayout.setSizeUndefined();
			
			addComponent(vertical);
			setModal(true);
			setResizable(false);
			
		}
		public void attach(){}

		private ComboBox getGroupsComboBox(){
			Integer firstItem=null;
			final ComboBox groupsCombo=new ComboBox(locale.getString("MESSAGELAYOUT_SEARCHWINDOW_GROUPSCOMBO_CAPTION"));
			groupsCombo.addListener(new Property.ValueChangeListener() {
				
				/**
				 * 
				 */
				private static final long serialVersionUID = 1L;

				@Override
				public void valueChange(ValueChangeEvent event) {

					
					QueryContainer container=storage.getStudentsContactInfoQueryContainer((Integer)groupsCombo.getValue());
					twincol.setContainerDataSource(container);
					twincol.setItemCaptionPropertyId("fullname");
				
				}
			});
			IndexedContainer ic=new IndexedContainer();
			ic.addContainerProperty("namegroup", String.class, null);
			Iterator<Group> it =getGroupsCollection().iterator();
			Item itemGroup=null;
			while (it.hasNext()){
				Group group=it.next();
				itemGroup=ic.addItem(group.getID());
				itemGroup.getItemProperty("namegroup").setValue(group.getName());
				if (firstItem==null){firstItem=group.getID();};
			}
			groupsCombo.setContainerDataSource(ic);
			groupsCombo.setItemCaptionPropertyId("namegroup");
			if (firstItem!=null){groupsCombo.setValue(firstItem);};
			return groupsCombo;
		}
	}
	private Collection<Group> getGroupsCollection(){
		if (adminmode){
			return storage.allGroups();
		}else {
			return storage.allGroups(app.user);
		}
	}

}
