/*
    AttendanceManager
    Copyright (C) 2010  Victor Carceler

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
 */

package org.iespuigcastellar.attendancemanager.screenlayouts;

import java.util.Collection;
import java.util.Date;
import java.util.Iterator;
import java.util.ResourceBundle;

import org.iespuigcastellar.attendancemanager.AttendancemanagerApplication;
import org.iespuigcastellar.attendancemanager.core.Classblock;
import org.iespuigcastellar.attendancemanager.core.DatabaseStorage;
import org.iespuigcastellar.attendancemanager.core.HasGetName;
import org.iespuigcastellar.attendancemanager.core.Logger;
import org.iespuigcastellar.attendancemanager.core.Miss;
import org.iespuigcastellar.attendancemanager.core.Student;

import com.vaadin.data.Item;
import com.vaadin.data.Property;
import com.vaadin.data.Property.ValueChangeEvent;
import com.vaadin.data.util.BeanItem;
import com.vaadin.terminal.ClassResource;
import com.vaadin.terminal.ThemeResource;
import com.vaadin.ui.AbstractSelect.Filtering;
import com.vaadin.ui.Button;
import com.vaadin.ui.Button.ClickEvent;
import com.vaadin.ui.Button.ClickListener;
import com.vaadin.ui.CheckBox;
import com.vaadin.ui.ComboBox;
import com.vaadin.ui.Embedded;
import com.vaadin.ui.Form;
import com.vaadin.ui.GridLayout;
import com.vaadin.ui.HorizontalLayout;
import com.vaadin.ui.Label;
import com.vaadin.ui.Panel;
import com.vaadin.ui.PopupDateField;
import com.vaadin.ui.Table;
import com.vaadin.ui.Table.CellStyleGenerator;
import com.vaadin.ui.TextField;
import com.vaadin.ui.VerticalLayout;
import com.vaadin.ui.Window;
import com.vaadin.ui.Window.Notification;
import com.vaadin.ui.themes.BaseTheme;

/**
 * Permite a los profesores pasar lista, consultar la asitencia y realizar cambios. Si
 * el usuario es administrador puede realizar estas operaciones en todos los grupos/clase
 * si no sólo en sus clases.
 * 
 * @author vcarceler
 */
@SuppressWarnings("serial")
public class TeacherMainLayout extends VerticalLayout implements Property.ValueChangeListener {

	private AttendancemanagerApplication app;
	
	private static ResourceBundle locale = AttendancemanagerApplication.locale;
	private static DatabaseStorage storage = AttendancemanagerApplication.storage;
	//private static DateFormat df = AttendancemanagerApplication.df;
//-----_>>>
	private Button changeLayoutButton;
	private Label stateMissLabel=new Label("");
	
	private PasswordChangeWindow passwordChangeWindow;
	private PopupDateField datePopupDateField;
	private ComboBox classblockComboBox;
	private Table table;

//--->>>
	private Classblock classblock;
	private Panel photoPanel;
	private GridLayout tablePhoto;
	private boolean adminMode = false;		// If true loads classblocks for all teachers
	
	

	/**
	 * Default constructor, builds an instance in normal mode showing only
	 * classblocks of teacher.
	 */
	public TeacherMainLayout() {
	}
	
	/**
	 * If adminMode == true, builds an instance in adminMode showing all
	 * classblocks of all teachers.
	 */
	public TeacherMainLayout(boolean adminMode) {
		this.adminMode = adminMode;
	}
	
	public void attach() {
		
		app = (AttendancemanagerApplication)getApplication();
		
		initLayout();
		
		Collection<Classblock> classblocks;
		
		if (adminMode) {
			classblocks = storage.classblocksOnDate(new Date());
		} else {
			classblocks = storage.classblocksOfTeacherOnDate(app.user.getID(), new Date());
		}
		
		setComboBoxItems(classblockComboBox, classblocks);
	}
	
	private void initLayout() {
		
		table = new Table();
		tablePhoto = new GridLayout();
		
		photoPanel = new Panel();
		photoPanel.setSizeFull();
		photoPanel.setImmediate(true);
		
		passwordChangeWindow = new PasswordChangeWindow();
		passwordChangeWindow.setCaption(locale.getString("TEACHERMAINLAYOUT_CHANGEMYPASSWORD_BUTTON"));
		
		datePopupDateField = new PopupDateField();
		datePopupDateField.setDescription(locale.getString("TEACHERMAINLAYOUT_DATEFIELD_DESCRIPTION"));
		datePopupDateField.setValue(new java.util.Date());
		datePopupDateField.setResolution(PopupDateField.RESOLUTION_DAY);
		datePopupDateField.setImmediate(true);
		datePopupDateField.addListener(ValueChangeEvent.class, this, "changedDate");
		
		classblockComboBox = new ComboBox();
		classblockComboBox.setInputPrompt(locale.getString("TEACHERMAINLAYOUT_CLASSBLOCK_INPUTPROMPT"));
		classblockComboBox.setDescription(locale.getString("TEACHERMAINLAYOUT_CLASSBLOCK_DESCRIPTION"));
		classblockComboBox.setFilteringMode(Filtering.FILTERINGMODE_CONTAINS);
		classblockComboBox.setImmediate(true);
		classblockComboBox.addContainerProperty("name", String.class, "");
		classblockComboBox.setItemCaptionPropertyId("name");
		classblockComboBox.addListener(ValueChangeEvent.class, this, "changedClassblock");
				
		Button logoutButton = new Button(locale.getString("TEACHERMAINLAYOUT_LOGOUTBUTTON_CAPTION"));
		
		logoutButton.addListener(new Button.ClickListener() {
			@Override
			public void buttonClick(ClickEvent event) {
				Logger.log("User " + app.user.getLogin() + " closes session");
				//storage.close();
				getApplication().close();
			}
		});
		
		Button changePasswordButton = new Button(locale.getString("TEACHERMAINLAYOUT_CHANGEMYPASSWORD_BUTTON"));
		changePasswordButton.setIcon(new ThemeResource("../runo/icons/16/user.png"));
		changePasswordButton.setStyleName( BaseTheme.BUTTON_LINK );
		changePasswordButton.addListener(new Button.ClickListener() {
			
			public void buttonClick(ClickEvent event) {
				// Open window if not open already
				if(passwordChangeWindow.getParent() != null) {
					// Window already open
				} else {
					// open window
					getWindow().addWindow(passwordChangeWindow);
				}
			}
		});
		
		HorizontalLayout optionsLayout = new HorizontalLayout();
		optionsLayout.setSpacing(true);
		optionsLayout.setWidth("100%");
		
		

		
		
		
		
		//optionsLayout.setComponentAlignment(changePasswordButton, Alignment.MIDDLE_LEFT);
		optionsLayout.addComponent(datePopupDateField);
		//optionsLayout.setComponentAlignment(datePopupDateField, Alignment.MIDDLE_LEFT);
		classblockComboBox.setWidth("80%");
		optionsLayout.addComponent(classblockComboBox);
		//optionsLayout.setComponentAlignment(classblockComboBox, Alignment.MIDDLE_LEFT);
//---------_>>>>		
		//botones de cabio de vista y añadir tipos de falta
		
		changeLayoutButton = new Button ();
		changeLayoutButton.setStyleName( BaseTheme.BUTTON_LINK);
		changeLayoutButton.setIcon(new ClassResource("screenlayouts/logininfo/grid.png", getApplication()));
		changeLayoutButton.setHeight(20);
		changeLayoutButton.setWidth(20);
		
		

		
		optionsLayout.addComponent(changeLayoutButton);

//---------->>		
		optionsLayout.setExpandRatio(classblockComboBox, 5.0f);
		optionsLayout.addComponent(changePasswordButton);
		optionsLayout.addComponent(logoutButton);
		//optionsLayout.setComponentAlignment(logoutButton, Alignment.MIDDLE_RIGHT);
		
		

		addComponent(optionsLayout);

		table.setSizeFull();
		table.setImmediate(true);
		table.setColumnReorderingAllowed(false);
	    table.setColumnCollapsingAllowed(true);
	    

//config tabla de fotos
//--------------------->>>
		tablePhoto.setSizeFull();
		tablePhoto.setImmediate(true);
		tablePhoto.setSpacing(true);
	
		photoPanel.addComponent(tablePhoto);
//----------_>>>		
		
		
	    table.addContainerProperty("Photo", Button.class, null);
	    table.addContainerProperty("Name", String.class, null);
	    table.addContainerProperty("Surname 1", String.class, null);
	    table.addContainerProperty("Surname 2", String.class, null);
	    table.addContainerProperty("Miss", CheckBox.class, null);
	    table.addContainerProperty("Excused", CheckBox.class, null);
	    table.addContainerProperty("Delay", CheckBox.class, null);
	    table.addContainerProperty("Expulsion", CheckBox.class, null);
	    table.addContainerProperty("Misc", CheckBox.class, null);
	    
	    
	    table.setColumnExpandRatio("Name", 1);
	    table.setColumnExpandRatio("Surname 1", 1);
	    table.setColumnExpandRatio("Surname 2", 1);

	    table.setColumnHeaders(new String[] { locale.getString("TEACHERMAINLAYOUT_TABLECOLUMN_PHOTO"), locale.getString("TEACHERMAINLAYOUT_TABLECOLUMN_NAME"), locale.getString("TEACHERMAINLAYOUT_TABLECOLUMN_SURNAME1"), locale.getString("TEACHERMAINLAYOUT_TABLECOLUMN_SURNAME2"), locale.getString("TEACHERMAINLAYOUT_TABLECOLUMN_MISS"), locale.getString("TEACHERMAINLAYOUT_TABLECOLUMN_EXCUSED"), locale.getString("TEACHERMAINLAYOUT_TABLECOLUMN_DELAY"), locale.getString("TEACHERMAINLAYOUT_TABLECOLUMN_EXPULSION"),"Misc" });
	    table.setStyleName("striped");
		addComponent(table);
		
		setExpandRatio(table, 1);
		setSizeFull();
		
//------------_>>>>>>>>>> evento cambio de tipo de vista	


		changeLayoutButton.addListener(new ClickListener(){

			@Override
			public void buttonClick(ClickEvent event) {
				// TODO Auto-generated method stub
				
				int indexOfTable = getComponentIndex(table);
				
				if (indexOfTable!=-1){
					
					// TABLA DE FOTOS
					//redresco de la tabla
					if (classblock!=null){
						loadTable(classblock.getIDGroup());
					}
					removeComponent(table);
					addComponent(photoPanel);
					setExpandRatio(photoPanel, 1);
					setSizeFull();
					changeLayoutButton.setIcon(new ClassResource("screenlayouts/logininfo/list.png", getApplication()));


				}
				else{
					//TABLA NORMAL
					//redresco de la tabla
					if (classblock!=null){
						loadTable(classblock.getIDGroup());
					}
					removeComponent(photoPanel);
					addComponent(table);
					setExpandRatio(table, 1);
					setSizeFull();
					changeLayoutButton.setIcon(new ClassResource("screenlayouts/logininfo/grid.png", getApplication()));

				}
	
				
			}
			
		});
		
	}
	
	

	
	
	/** Clears ComboBox's Items and sets the ones in collection.  */
	private void setComboBoxItems(ComboBox combo, Collection<?> collection) {
		combo.removeAllItems();
		for(Iterator<?> it = collection.iterator(); it.hasNext(); ) {
			HasGetName element = (HasGetName) it.next();
			Item item = combo.addItem(element);
			item.getItemProperty("name").setValue(element.getName());
		}
		// If there is only one Item select it
		if(collection.size()==1) {
			combo.select(collection.iterator().next());
		}
	}

    public void valueChange(ValueChangeEvent event) {
        getWindow().showNotification("Subject: " + event.getProperty());
    }
    
    public void changedClassblock(ValueChangeEvent event) {
 ///-----_>>>retorno de classbock en global para refrscar lista en cambio de layout   	
    	classblock = (Classblock)event.getProperty().getValue();
    	if(classblock != null) {
    		loadTable(classblock.getIDGroup());
			// style generator
//----_>>>>>
    		setTableStyleMissRow();
	        
	        
	        
    	} else {
    		table.removeAllItems();
//---Eliminamos componentes de la tablephoto 
    		tablePhoto.removeAllComponents();
    		getWindow().showNotification(locale.getString("TEACHERMAINLAYOUT_SELECTCLASSBLOCKNOTIFICATION_CAPTION"), "<br>"+locale.getString("TEACHERMAINLAYOUT_SELECTCLASSBLOCKNOTIFICATION_DESCRIPTION"), Notification.TYPE_HUMANIZED_MESSAGE);
    	}
    }
    
    public void changedDate(ValueChangeEvent event) {
    	table.removeAllItems();
    	//---Eliminamos componentes de la tablephoto 
		tablePhoto.removeAllComponents();
		
		Collection<Classblock> classblocks;
		Date date = (Date)event.getProperty().getValue();
		
		if (adminMode) {
			classblocks = storage.classblocksOnDate(date);
			//System.out.println("Admin mode !!!");
		} else {
			classblocks = storage.classblocksOfTeacherOnDate(app.user.getID(), date);
			//System.out.println("NO admin mode !!!");
		}
    	
    	setComboBoxItems(classblockComboBox, classblocks);
    }
    
    
//-------------------------------_>>>>
    //carga estudiantes en tabla
    /** Loads students data into table. */
    public void loadTable(int idgroup) {
    	table.removeAllItems();
    	tablePhoto.removeAllComponents();
    	
    	
    	//coleccion de estudiantes del grupo inicio del bucle para cada estudiante
    	Collection<Student> collection = storage.studentsOfGroup(idgroup);
    	
 //------>> definir columnas filas de la grid de fotos alumnos
    	Embedded img = null;
    	int colum = 0;
    	int rowgrid=0;
    	tablePhoto.setColumns(4);
    	int sizeRows = collection.size()/4;
    	tablePhoto.setRows(sizeRows+2);
    	
    	
    	for(Iterator<Student> it = collection.iterator(); it.hasNext(); ) {
			final Student student = it.next();
		
			
			CheckBox missCheckBox = new CheckBox();
			missCheckBox.setData(student);
			if(storage.existsMiss(Miss.MISS, (Date)datePopupDateField.getValue(), ((Classblock)classblockComboBox.getValue()).getID(), student.getID())) {
				missCheckBox.setValue(true);
			}
			missCheckBox.setImmediate(true);
			missCheckBox.addListener(new Button.ClickListener() {
				
				@Override
				public void buttonClick(ClickEvent event) {
					Student student = (Student)event.getButton().getData();
					
					Miss miss = new Miss(0, Miss.MISS, (Date)datePopupDateField.getValue(), "", student.getID(), ((Classblock)classblockComboBox.getValue()).getIDTeacher(), ((Classblock)classblockComboBox.getValue()).getIDSubject(), ((Classblock)classblockComboBox.getValue()).getID());

					if((Boolean)event.getButton().getValue()) {
						storage.addMiss(miss);
					} else {
						// Delete Miss
						storage.deleteMiss(Miss.MISS, miss.getDate(), miss.getIDStudent(), miss.getIDClassblock());
					}
					updateEnabledCheckBoxes(student);
				}
			});
			
			CheckBox excusedmissCheckBox = new CheckBox();
			excusedmissCheckBox.setData(student);
			if(storage.existsMiss(Miss.EXCUSED_MISS, (Date)datePopupDateField.getValue(), ((Classblock)classblockComboBox.getValue()).getID(), student.getID())) {
				excusedmissCheckBox.setValue(true);
			}
			excusedmissCheckBox.setImmediate(true);
			excusedmissCheckBox.addListener(new Button.ClickListener() {
				
				@Override
				public void buttonClick(ClickEvent event) {
					Student student = (Student)event.getButton().getData();
					
					Miss miss = new Miss(0, Miss.EXCUSED_MISS, (Date)datePopupDateField.getValue(), "", student.getID(), ((Classblock)classblockComboBox.getValue()).getIDTeacher(), ((Classblock)classblockComboBox.getValue()).getIDSubject(), ((Classblock)classblockComboBox.getValue()).getID());

					if((Boolean)event.getButton().getValue()) {
						// New Miss
						storage.addMiss(miss);
					} else {
						// Delete Miss
						storage.deleteMiss(Miss.EXCUSED_MISS, miss.getDate(), miss.getIDStudent(), miss.getIDClassblock());
					}
					updateEnabledCheckBoxes(student);
				}
			});
			
			CheckBox delayCheckBox = new CheckBox();
			delayCheckBox.setData(student);
			if(storage.existsMiss(Miss.DELAY, (Date)datePopupDateField.getValue(), ((Classblock)classblockComboBox.getValue()).getID(), student.getID())) {
				delayCheckBox.setValue(true);
			}
			delayCheckBox.setImmediate(true);
			delayCheckBox.addListener(new Button.ClickListener() {
				
				@Override
				public void buttonClick(ClickEvent event) {
					Student student = (Student)event.getButton().getData();
					Miss miss = new Miss(0, Miss.DELAY, (Date)datePopupDateField.getValue(), "", student.getID(), ((Classblock)classblockComboBox.getValue()).getIDTeacher(), ((Classblock)classblockComboBox.getValue()).getIDSubject(), ((Classblock)classblockComboBox.getValue()).getID());

					if((Boolean)event.getButton().getValue()) {
						storage.addMiss(miss);
					} else {
						storage.deleteMiss(Miss.DELAY, miss.getDate(), miss.getIDStudent(), miss.getIDClassblock());
						
					}
					updateEnabledCheckBoxes(student);
				}
			});
			
			CheckBox expulsionCheckBox = new CheckBox();
			expulsionCheckBox.setData(student);
			if(storage.existsMiss(Miss.EXPULSION, (Date)datePopupDateField.getValue(), ((Classblock)classblockComboBox.getValue()).getID(), student.getID())) {
				expulsionCheckBox.setValue(true);
			}
			expulsionCheckBox.setImmediate(true);
			expulsionCheckBox.addListener(new Button.ClickListener() {
				
				@Override
				public void buttonClick(ClickEvent event) {
					Student student = (Student)event.getButton().getData();
					Miss miss = new Miss(0, Miss.EXPULSION, (Date)datePopupDateField.getValue(), "", student.getID(), ((Classblock)classblockComboBox.getValue()).getIDTeacher(), ((Classblock)classblockComboBox.getValue()).getIDSubject(), ((Classblock)classblockComboBox.getValue()).getID());

					if((Boolean)event.getButton().getValue()) {
						storage.addMiss(miss);
					} else {
						storage.deleteMiss(Miss.EXPULSION, miss.getDate(), miss.getIDStudent(), miss.getIDClassblock());
					}
					updateEnabledCheckBoxes(student);
				}
			});
////------>>>
//añadido checkbox de miscelanea
			CheckBox miscCheckBox = new CheckBox();
			miscCheckBox.setData(student);
			if(storage.existsMiss(Miss.MISC, (Date)datePopupDateField.getValue(), ((Classblock)classblockComboBox.getValue()).getID(), student.getID())) {
				miscCheckBox.setValue(true);
			}
			miscCheckBox.setImmediate(true);
			miscCheckBox.addListener(new Button.ClickListener() {
				
				@Override
				public void buttonClick(ClickEvent event) {
					Student student = (Student)event.getButton().getData();
					Miss miss = new Miss(0, Miss.MISC, (Date)datePopupDateField.getValue(), "", student.getID(), ((Classblock)classblockComboBox.getValue()).getIDTeacher(), ((Classblock)classblockComboBox.getValue()).getIDSubject(), ((Classblock)classblockComboBox.getValue()).getID());

					if((Boolean)event.getButton().getValue()) {
						storage.addMiss(miss);
					} else {
						storage.deleteMiss(Miss.MISC, miss.getDate(), miss.getIDStudent(), miss.getIDClassblock());
					}
					updateEnabledCheckBoxes(student);
				}
			});
//--------_>>>			
			//construccionde objeto en la row
			//primer componente contructor de icono para la foto correspondiente de cada estudiante
			Button camPhoto = new Button();
			camPhoto.setHeight(20);
			camPhoto.setWidth(20);
			camPhoto.setEnabled(false);
//--->>
			//metodos definidos al final de la clase, a traves de un student genera su Blob para retornar una aray de bytes
			//creamos una imagen en cache que se sobreescribe para cada alumno
		
			
			if (student.getBytePhotoArray()!=null){
				//recojemos la localizacion relativa de la foto en cache
				String image =  app.getRelativeLocation(student.getImageResource(getApplication(), Integer.toString(student.getID())+".png"));
				camPhoto.setDescription("<h2><img src=\""+image+"\"/></h2>"
		                + "<br/>"+student.getName()+" "+ student.getSurname1());
				camPhoto.setIcon(new ThemeResource("../runo/icons/16/user.png"));
			}
			//--->>


			
			//nueva array de objetos de fila y añadido a la tabla
			Object row[] = { camPhoto, student.getName(), student.getSurname1(), student.getSurname2(), missCheckBox, excusedmissCheckBox, delayCheckBox, expulsionCheckBox,miscCheckBox};
			
			table.addItem(row, new Integer(student.getID()));
			

//---->>			
			//creacion de tablePhoto			
			final Panel studentBox = new Panel();	
			final Button studentButton = new Button();
		
			
			
			studentButton.setIcon(new ClassResource("screenlayouts/logininfo/User.png", app));
			studentButton.setSizeFull();
			//filtramos si los estudiantes contienen foto o no, si no es asi , imagen por defecto
			if (student.getBytePhotoArray()!=null){
			
				studentButton.setIcon(student.getImageResource(getApplication(), Integer.toString(student.getID())));
			}
			
			//deteccion de colores del studentButton
			if(storage.existsMiss(Miss.MISS, (Date)datePopupDateField.getValue(), ((Classblock)classblockComboBox.getValue()).getID(), student.getID())) {
				studentButton.setStyleName("buttonMiss");
				stateMissLabel.setCaption("Falta");
				
			}
			else if(storage.existsMiss(Miss.DELAY, (Date)datePopupDateField.getValue(), ((Classblock)classblockComboBox.getValue()).getID(), student.getID())) {
				studentButton.setStyleName("buttonDelay");
				stateMissLabel.setCaption("Rerás");
			}
			else if(storage.existsMiss(Miss.EXCUSED_MISS, (Date)datePopupDateField.getValue(), ((Classblock)classblockComboBox.getValue()).getID(), student.getID())) {
				studentButton.setStyleName("buttonExcused");
				stateMissLabel.setCaption("Justificació");
			}
			else if(storage.existsMiss(Miss.EXPULSION, (Date)datePopupDateField.getValue(), ((Classblock)classblockComboBox.getValue()).getID(), student.getID())) {
				studentButton.setStyleName("buttonExpulsion");
				stateMissLabel.setCaption("Expulsió");
			}
			else{
				studentButton.setStyleName("buttonNone");
				stateMissLabel.setCaption("");
			}
			
			//añadimos al studentBox
			VerticalLayout separator = new VerticalLayout();

			separator.addComponent(studentButton);
//			separator.addComponent(stateMissLabel);
			studentBox.addComponent(separator);

		
			studentBox.setCaption(student.getFullName());
	
			//envento del studentBox
			studentButton.addListener(new ClickListener(){
				@Override
				public void buttonClick(ClickEvent event) {
					// TODO Auto-generated method stub

					refreshPhotoMiss(student,studentButton);
					studentButton.requestRepaint();					
				}
				
			});
			

			
			//añadimos a la tabla
			tablePhoto.addComponent(studentBox,colum,rowgrid);
			
//--->>
			//cambiamos posicion columna
			colum++;
			if (colum ==4){
				colum=0;
				rowgrid++;
			}
			
			
			updateEnabledCheckBoxes(student);
		}
    	tablePhoto.isImmediate();

    }

 //---->>>>
    //METODO REFRESCO DEL COLOR DE LOS BOTONES QUE REACCIONA AL EVENTO DE CLICK DEL BOTON
    //deèndiendo del Miss de ese Student el click reaccionara añadiendo o eliminando las miss y cambiando el color
    //correspondiente
   private void refreshPhotoMiss(Student student, Button studentButton){
		//añadimos falta

		//añadimos retras
		if(storage.existsMiss(Miss.MISS, (Date)datePopupDateField.getValue(), ((Classblock)classblockComboBox.getValue()).getID(), student.getID())) {
			
			//Esta en CSS pendiente de especificar RGB en codigo
			studentButton.setStyleName("buttonDelay");
			stateMissLabel.setCaption("Retrás");
			//añadimos retraso al student y eliminamos los otros tipos de falta que pueda tener
			Miss miss = new Miss(0, Miss.DELAY, (Date)datePopupDateField.getValue(), "", student.getID(), ((Classblock)classblockComboBox.getValue()).getIDTeacher(), ((Classblock)classblockComboBox.getValue()).getIDSubject(), ((Classblock)classblockComboBox.getValue()).getID());
			storage.addMiss(miss);
			storage.deleteMiss(Miss.EXCUSED_MISS, miss.getDate(), miss.getIDStudent(), miss.getIDClassblock());
			storage.deleteMiss(Miss.EXPULSION, miss.getDate(), miss.getIDStudent(), miss.getIDClassblock());
			storage.deleteMiss(Miss.MISS, miss.getDate(), miss.getIDStudent(), miss.getIDClassblock());
///----->>>>>>>>>>>
//CHANGE LOCATE			
			//mostramos por pantalla
			getParent().getWindow().showNotification("Retrás" + getStyleName(),Notification.TYPE_TRAY_NOTIFICATION);
			

		}
		else if(storage.existsMiss(Miss.DELAY, (Date)datePopupDateField.getValue(), ((Classblock)classblockComboBox.getValue()).getID(), student.getID())) {
			//Esta en CSS pendiente de especificar RGB en codigo
			studentButton.setStyleName("buttonExcused");
			stateMissLabel.setCaption("Justficació");
			//añadimos expulsion al student
			Miss miss = new Miss(0, Miss.EXCUSED_MISS, (Date)datePopupDateField.getValue(), "", student.getID(), ((Classblock)classblockComboBox.getValue()).getIDTeacher(), ((Classblock)classblockComboBox.getValue()).getIDSubject(), ((Classblock)classblockComboBox.getValue()).getID());
			storage.addMiss(miss);
			//retiramos otros tipos de falta
			storage.deleteMiss(Miss.EXPULSION, miss.getDate(), miss.getIDStudent(), miss.getIDClassblock());
			storage.deleteMiss(Miss.DELAY, miss.getDate(), miss.getIDStudent(), miss.getIDClassblock());
			storage.deleteMiss(Miss.MISS, miss.getDate(), miss.getIDStudent(), miss.getIDClassblock());
///----->>>>>>>>>>>
//CHANGE LOCATE		
			//mostramos por pantalla
			getParent().getWindow().showNotification("Justificada" + getStyleName(),Notification.TYPE_TRAY_NOTIFICATION);
		
			
			
		}//no tiene falta
		//expulsion
		else if(storage.existsMiss(Miss.EXCUSED_MISS, (Date)datePopupDateField.getValue(), ((Classblock)classblockComboBox.getValue()).getID(), student.getID())) {
			//Esta en CSS pendiente de especificar RGB en codigo
			studentButton.setStyleName("buttonExpulsion");
			stateMissLabel.setCaption("Expulsió");
			//añadimos expulsion al student
			Miss miss = new Miss(0, Miss.EXPULSION, (Date)datePopupDateField.getValue(), "", student.getID(), ((Classblock)classblockComboBox.getValue()).getIDTeacher(), ((Classblock)classblockComboBox.getValue()).getIDSubject(), ((Classblock)classblockComboBox.getValue()).getID());
			storage.addMiss(miss);
			//retiramos otros tipos de falta
			storage.deleteMiss(Miss.EXCUSED_MISS, miss.getDate(), miss.getIDStudent(), miss.getIDClassblock());
			storage.deleteMiss(Miss.DELAY, miss.getDate(), miss.getIDStudent(), miss.getIDClassblock());
			storage.deleteMiss(Miss.MISS, miss.getDate(), miss.getIDStudent(), miss.getIDClassblock());
///----->>>>>>>>>>>
//CHANGE LOCATE
			//mostramos por pantalla
			getParent().getWindow().showNotification("Expulsió" + getStyleName(),Notification.TYPE_TRAY_NOTIFICATION);
		
			
		}//no tiene falta
	
	
		else if(storage.existsMiss(Miss.EXPULSION, (Date)datePopupDateField.getValue(), ((Classblock)classblockComboBox.getValue()).getID(), student.getID())) {
			
			//Esta en CSS pendiente de especificar RGB en codigo
			studentButton.setStyleName("buttonNone");
			stateMissLabel.setCaption("");
			//en el ultimo click lo unico que habra adquirido antes es Expulsion
			Miss miss = new Miss(0, Miss.EXPULSION, (Date)datePopupDateField.getValue(), "", student.getID(), ((Classblock)classblockComboBox.getValue()).getIDTeacher(), ((Classblock)classblockComboBox.getValue()).getIDSubject(), ((Classblock)classblockComboBox.getValue()).getID());
			storage.deleteMiss(Miss.EXPULSION, miss.getDate(), miss.getIDStudent(), miss.getIDClassblock());
			storage.deleteMiss(Miss.EXCUSED_MISS, miss.getDate(), miss.getIDStudent(), miss.getIDClassblock());
			storage.deleteMiss(Miss.MISS, miss.getDate(), miss.getIDStudent(), miss.getIDClassblock());
			storage.deleteMiss(Miss.DELAY, miss.getDate(), miss.getIDStudent(), miss.getIDClassblock());

			//mostramos por pantalla
//			getParent().getWindow().showNotification("Nothing --->" + getStyleName(),Notification.TYPE_TRAY_NOTIFICATION);
			
		}//falta
		else{
			//Esta en CSS pendiente de especificar RGB en codigo
			studentButton.setStyleName("buttonMiss");
			stateMissLabel.setCaption("Falta");
			//añadimos falta al student
			Miss miss = new Miss(0, Miss.MISS, (Date)datePopupDateField.getValue(), "", student.getID(), ((Classblock)classblockComboBox.getValue()).getIDTeacher(), ((Classblock)classblockComboBox.getValue()).getIDSubject(), ((Classblock)classblockComboBox.getValue()).getID());
			storage.addMiss(miss);
			//mostramos estado
///----->>>>>>>>>>>
//CHANGE LOCATE
			getParent().getWindow().showNotification("Falta" + getStyleName(),Notification.TYPE_TRAY_NOTIFICATION);						
			studentButton.setValidationVisible(false);
		}
   }
    
    
    /* Updates the enabled/disabled state of checkboxes for a student */
    private void updateEnabledCheckBoxes(Student student) {
    	boolean missFlag = true;
    	boolean excusedFlag = true;
    	boolean delayFlag = true;
    	boolean expulsionFlag = true;
    	boolean miscFlag = true;
    	
    	CheckBox missCheckBox = (CheckBox)table.getItem(student.getID()).getItemProperty("Miss").getValue();
    	if((Boolean)missCheckBox.getValue()) {
    		excusedFlag = false;
    		delayFlag = false;
    		expulsionFlag = false;
    	}
    	
    	CheckBox excusedCheckBox = (CheckBox)table.getItem(student.getID()).getItemProperty("Excused").getValue();
    	if((Boolean)excusedCheckBox.getValue()) {
    		missFlag = false;
    		delayFlag = false;
    		expulsionFlag = false;
    	}
    	
    	CheckBox delayCheckBox = (CheckBox)table.getItem(student.getID()).getItemProperty("Delay").getValue();
    	if((Boolean)delayCheckBox.getValue()) {
    		missFlag = false;
    		excusedFlag = false;
    	}
    	
    	CheckBox expulsionCheckBox = (CheckBox)table.getItem(student.getID()).getItemProperty("Expulsion").getValue();
    	if((Boolean)expulsionCheckBox.getValue()) {
    		missFlag = false;
    		excusedFlag = false;
    	}
    	
    	
    	CheckBox miscCheckBox = (CheckBox)table.getItem(student.getID()).getItemProperty("Misc").getValue();
    	if((Boolean)miscCheckBox.getValue()) {
    		missFlag = false;
    		excusedFlag = false;
    		delayFlag = false;
    		expulsionFlag = false;
    	}
    	
    	if(missFlag != missCheckBox.isEnabled()) missCheckBox.setEnabled(missFlag);
    	if(excusedFlag != excusedCheckBox.isEnabled()) excusedCheckBox.setEnabled(excusedFlag);
    	if(delayFlag != delayCheckBox.isEnabled()) delayCheckBox.setEnabled(delayFlag);
    	if(expulsionFlag != expulsionCheckBox.isEnabled()) expulsionCheckBox.setEnabled(expulsionFlag);
    	if(miscFlag != miscCheckBox.isEnabled()) miscCheckBox.setEnabled(miscFlag);

    }
    
    private class PasswordChangeWindow extends Window {
    	
    	private final Form passwordForm = new Form();
    	private final PasswordInfo passwordInfo = new PasswordInfo();
    	
    	
    	public class PasswordInfo {
    		String passwordA = "";
    		String passwordB = "";
    		
			@SuppressWarnings("unused")
			public String getPasswordA() {
				return passwordA;
			}
			@SuppressWarnings("unused")
			public void setPasswordA(String passwordA) {
				this.passwordA = passwordA;
			}
			@SuppressWarnings("unused")
			public String getPasswordB() {
				return passwordB;
			}
			@SuppressWarnings("unused")
			public void setPasswordB(String passwordB) {
				this.passwordB = passwordB;
			}
    		
    		
    	}
    	
    	public PasswordChangeWindow() {
    		
    		BeanItem<PasswordInfo> passwordBean = new BeanItem<PasswordInfo>(passwordInfo);
    		passwordForm.setItemDataSource(passwordBean);
    		
    		TextField tf = (TextField)passwordForm.getField("passwordA");
    		
    		tf.setRequired(true);
    		tf.setCaption(locale.getString("TEACHERMAINLAYOUT_PASSWORDA_CAPTION"));
    		tf.setRequiredError(locale.getString("TEACHERMAINLAYOUT_PASSWORDA_REQUIREDERROR"));
    		tf.setSecret(true);
    		
    		tf = (TextField)passwordForm.getField("passwordB");
    		
    		tf.setRequired(true);
    		tf.setCaption(locale.getString("TEACHERMAINLAYOUT_PASSWORDB_CAPTION"));
    		tf.setRequiredError(locale.getString("TEACHERMAINLAYOUT_PASSWORDB_REQUIREDERROR"));
    		tf.setSecret(true);
    		
    		passwordForm.setImmediate(true);
    		
    		Button okButton = new Button(locale.getString("GROUPMANAGERLAYOUT_BUTTONOK_CAPTION"));
    		okButton.setIcon(new ThemeResource("../runo/icons/16/ok.png"));
    		okButton.addListener(new Button.ClickListener() {
				
				@Override
				public void buttonClick(ClickEvent event) {
					Window parentWindow = (Window) getWindow().getParent();
					if(passwordInfo.passwordA.equals(passwordInfo.passwordB)) {
						app.user.setPassword(storage.getPasswordHash(passwordInfo.passwordA));
						storage.updateTeacher(app.user);
						parentWindow.showNotification(locale.getString("TEACHERMAINLAYOUT_PASSWORDCHANGEOKNOTIFICATION_CAPTION"), locale.getString("TEACHERMAINLAYOUT_PASSWORDCHANGEOKNOTIFICATION_DESCRIPTION"), Notification.TYPE_TRAY_NOTIFICATION);
						
						// Close window
						parentWindow.removeWindow(passwordChangeWindow);
						
						passwordForm.getField("passwordA").setValue("");
						passwordForm.getField("passwordB").setValue("");
					} else {
						parentWindow.showNotification(locale.getString("TEACHERMAINLAYOUT_PASSWORDCHANGEERRORNOTIFICATION_CAPTION"), locale.getString("TEACHERMAINLAYOUT_PASSWORDCHANGEERRORNOTIFICATION_DESCRIPTION"), Notification.TYPE_WARNING_MESSAGE);
					}
					
				}
			});
    		
    		passwordForm.getFooter().addComponent(okButton);
    		
    		Button cancelButton = new Button(locale.getString("GROUPMANAGERLAYOUT_BUTTONCANCEL_CAPTION"));
    		cancelButton.setIcon(new ThemeResource("../runo/icons/16/cancel.png"));
    		cancelButton.addListener(new Button.ClickListener() {
				
				@Override
				public void buttonClick(ClickEvent event) {
					// Close window
					((Window) passwordChangeWindow.getParent()).removeWindow(passwordChangeWindow);
					passwordForm.getField("passwordA").setValue("");
					passwordForm.getField("passwordB").setValue("");
				}
			});
    		
    		passwordForm.getFooter().addComponent(cancelButton);
    		
    		addComponent(passwordForm);
    		
    		VerticalLayout layout = (VerticalLayout) getContent();
    		layout.setSpacing(true);
    		layout.setMargin(true);
    		layout.setSizeUndefined(); // We want the window to has automatic size !!!
    		
    		setModal(true);
    		setResizable(false);
    	}
    }
   
    //Genera el estilo de las rows que tengan algun tipo de falta la hora anterior
    public void setTableStyleMissRow(){
    	//---------->>>>> refresco de las celdas
        table.setCellStyleGenerator(new CellStyleGenerator() {
        	public String getStyle(Object itemId, Object propertyId) {
	                if (app.storage.isPreviousMiss(Miss.MISS, classblock,(Integer) itemId, 1)){
	                	return "markedMiss";
	                }
	                else if (app.storage.isPreviousMiss(Miss.EXCUSED_MISS, classblock,(Integer) itemId, 1)){
	                	return "markedExcused";
	                }
	                else if (app.storage.isPreviousMiss(Miss.DELAY, classblock,(Integer) itemId, 1)){
	                	return "markedDelay";
	                }
	                else if (app.storage.isPreviousMiss(Miss.EXPULSION, classblock,(Integer) itemId, 1)){
	                	return "markedExpulsion";
	                }		                
	                else{
	                    // no style
	                	return null;
	                }
            }
        });
        table.requestRepaint();
    }
}

